
/**
 * Expert em resolver automaticamente jogadas NIM.
 * Implementa o padro Strategy.
 * @stereotype Model - Strategy
 */
abstract class Expert
{
  static final int WINNER_EXPERT     = 0;
  static final int DUMMY_EXPERT      = 1;

  static final String[] NAMES = { "Implacavel", "Meigo" };

  abstract void computerTurn( int[] rows, Selection selection );

  /**
   * O factory: cria uma instncia a partir de um id.
   * Desacopla o resto do sistema dos algoritmos utilizados.
   *
   * @param O indice associado ao elemento em getNames().
   */
  static Expert createInstance( int id )
  {
    Expert result = null;

    if ( id == DUMMY_EXPERT )
    {
      result = new DummyExpert();
    }
    else if ( id == WINNER_EXPERT )
    {
      result = new WinnerExpert();
    }
    else
      throw new IllegalArgumentException( "Expert no reconhecido: " + id );

    return result;
  }


  /**
   * Retorna os nomes dos Experts definidos.
   */
  static String[] getExpertNames()
  {
    return NAMES;
  }

  static String getExpertName( int id )
  {
    return NAMES[ id ];
  }

}



/**
 * O dummy expert.
 */
class DummyExpert extends Expert
{
  void computerTurn( int[] rows, Selection selection )
  {
    for ( int i = 0; i < rows.length; i++ )
    {
      if ( rows[ i ] > 0 )
      {
        selection.selectRow( i );
        selection.togglePiece( 0 );
        break;
      }
    } // for
  }
} // class DummyExpert



/**
 * O expert que ganha sempre.
 */
class WinnerExpert extends Expert
{
  void computerTurn( int[] rows, Selection selection )
  {
    int[] move = compMove( rows );
    selection.selectRow( move[ 0 ] );

    for ( int i = 0; i < move[ 1 ]; i++ )
    {
      selection.togglePiece( i );
    }
  }

  private int[] compMove( int[] sizeP )
  {
    int nP = sizeP.length;

    int cmove[] = new int[3];
    int binary[][] = new int[6][4];
    int power[] = new int[4];
    int t[] = new int[4];
    int f = 0;
    int j = -1;
    power[0] = 1;
    power[1] = 2;
    power[2] = 4;
    power[3] = 8;
    int i;
    for (i = 0; i < nP; i++)
    {
      int sP = sizeP[i];
      j = 3;
      do
        if (sP < power[j])
        {
          binary[i][j] = 0;
        }
        else
        {
          binary[i][j] = 1;
          sP -= power[j];
        }
      while (--j >= 0);
    }

    i = 0;
    do
    {
      for (j = 0; j < nP; j++)
        t[i] = t[i] + binary[j][i];

    } while (++i < 4);
    i = 0;
    do
      if (2 * (t[i] / 2) != t[i])
        f = i;
    while (++i < 4);
    for (j = 0; j < nP; j++)
      if (binary[j][f] == 1)
        cmove[0] = j;

    j = 3;
    do
    {
      if (binary[cmove[0]][j] == 1 && 2 * (t[j] / 2) != t[j])
        cmove[1] = cmove[1] + power[j];
      if (binary[cmove[0]][j] == 0 && 2 * (t[j] / 2) != t[j])
        cmove[1] = cmove[1] - power[j];
    } while (--j >= 0);
    for (j = 0; cmove[1] == 0; j++)
      if (sizeP[j] > 0)
      {
        cmove[0] = j;
        ////cmove[1] = 1 + (int)(Math.random() * (double)sizeP[j]);
        cmove[1] = 1 + 0;
      }
    return cmove;
  }
} // class WinnerExpert



/**
 * O expert do film...
 */
class MariembadExpert extends Expert
{
  private static final String[] PATTERNS =
  {
    "1155", "1144", "1133", "1122",
    "111", "123", "145", "246",
    "22", "33", "44", "55"
  };

  void computerTurn( int[] rows, Selection selection )
  {
    int[] newRows = new int[ rows.length ];
    int nextIndex = 0;

    // Copia e ordena o vetor.
    System.arraycopy( rows, 0, newRows, 0, rows.length );
    sort( newRows );

    boolean isChecked = false;
    for ( int p = 0; p < PATTERNS.length; p++ )
    {
      isChecked = checkPattern( PATTERNS[ p ], newRows, selection );
      if ( isChecked )
      {
        break;
      }
    } // for p

    if ( !isChecked )
    {
      new DummyExpert().computerTurn( rows, selection );
    }
  }


  private boolean checkPattern( String pattern, int[] rows, Selection selection )
  {
    boolean solved = false;
    boolean existsGreater = false;

    for ( int i = 0; i < pattern.length(); i++ )
    {
      int val = Character.digit( pattern.charAt( i ), 10 );

      if ( val < rows[ i ]  )
      {
        if ( existsGreater )
          break;
        else
        {
          existsGreater = true;
          selection.selectRow( i );
        }
      }
      else if ( val > rows[ i ] )
      {
        break;
      }
      else //if ( val == rows[ i ] )
        ; // nothing
    }

    return existsGreater && solved;
  }


  private void sort( int[] a )
  {
    int temp;// = 0;

    // Bubble sort.
    for ( int j = a.length - 1; j > 0; j-- )
    {
      for ( int i = 0; i <= j; i++ )
      {
        if ( a[ i ] > a[ i + 1 ] )
        {
          temp = a[ i ];
          a[ i ] = a[ i + 1 ];
          a[ i + 1 ] = temp;
        }
      } // for i
    } // for j
  }

} // class MariembadExpert


