import javax.microedition.midlet.*;
import javax.microedition.lcdui.*;
import java.util.*;

/**
 * Gerenca a representao grafica do jogo e a manipulao de eventos.
 * @stereotype UI
 */
class NimCanvas extends Canvas
  implements CommandListener
{
  private static final int Y_OFFSET_PIECES  = 35;
  private static final int X_OFFSET_PIECES  = 10;
  private static final int PIECE_SIDE       = 10;
  private static final int PIECE_DISTANCE   = 5;

  /**
   * @link aggregationByValue
   * @supplierCardinality 1
   * @supplierRole game 
   */
  private NimGame game;

  /**
   * @supplierRole theMidlet
   * @supplierCardinality 1 
   */
  private MainMIDlet theMidlet;
  private Command exitCommand;
  private Command chooseCommand;
  private Command goBackCommand;

  /**
   * A seleo atual do usurio humano.
   * @link aggregationByValue
   * @supplierCardinality 1
   * @supplierRole selection
   */
  private Selection selection;


  public NimCanvas( MainMIDlet theMidlet )
  {
    this.theMidlet  = theMidlet;
    this.game       = new NimGame();
    this.selection  = new Selection();

    // Comandos fixos.
    this.exitCommand   = new Command( "Sair", Command.EXIT, 2 );
    this.goBackCommand = new Command( "Voltar", Command.SCREEN, 2 );

    // Ser setado em startNewGame.
    this.chooseCommand = null;

    addCommand( this.exitCommand );
    addCommand( this.goBackCommand );
    setCommandListener( this );
  }


  public void startNewGame( Expert expert )
  {
    this.game.startNewGame( expert );
    this.selection.unselectAll();

    // Dependendo de quem  o proximo jogador, prepara a tela.
    setCurrentPlayerCommand();
  }


  public void commandAction( Command c, Displayable d )
  {
    if ( c == this.exitCommand )
    {
      this.theMidlet.destroyApp( true );
      this.theMidlet.notifyDestroyed();
    }
    else if ( c == this.chooseCommand )
    {
      // Se este evento ocorriou, quer dizer que existe o command, e portanto,
      // que o jogo ainda no acabou.
      this.selection = this.game.play( this.selection );;

      // Dependendo de quem  o proximo jogador, prepara a tela.
      setCurrentPlayerCommand();

      repaint();
    }
    else if ( c == this.goBackCommand )
    {
      this.theMidlet.goToMainForm();
    }
  }


  protected void paint( Graphics g )
  {
    final int width   = getWidth();
    final int height  = getHeight();

    // Clean background.
    g.setColor( 100, 100, 100 );
    g.fillRect( 0, 0, width, height );

    // Draw all rows and selection.
    int yOffset = Y_OFFSET_PIECES;
    for ( int row = 0; row < this.game.getNumberOfRows(); row++ )
    {
      int nPieces = this.game.getNumberOfPiecesInRow( row );

      // Draw row.
      int xOffset = X_OFFSET_PIECES;
      for ( int col = 0; col < nPieces; col++ )
      {
        if ( this.selection.isSelected( row, col ) )
        {
          if ( this.game.currentPlayerIsPalm() )
          {
            g.setColor( 255, 255, 255 );
          }
          else
          {
            g.setColor( 200, 200, 10 );
          }
        }
        else
        {
          g.setColor( 255, 100, 0 );
        }

        g.fillRect( xOffset, yOffset, PIECE_SIDE, PIECE_SIDE );

        g.setColor( 0, 100, 100 );
        g.drawRect( xOffset, yOffset, PIECE_SIDE, PIECE_SIDE );

        xOffset += ( PIECE_SIDE + PIECE_DISTANCE );
      } // for piece

      yOffset += ( PIECE_SIDE + PIECE_DISTANCE );
    } // for row

    // Draw white message over a gray band.
    g.setColor( 150, 150, 150 );
    g.fillRect( 0, 0, width, g.getFont().getHeight() );
    g.setColor( 255, 255, 255 );
    g.drawString( this.game.getCurrentMessage(), width / 2, 0, Graphics.TOP|Graphics.HCENTER );
  }


  protected void pointerPressed( int x, int y )
  {
    if ( !this.game.isTerminated() )
    {
      // Select clicked row.
      int row = (y - Y_OFFSET_PIECES) / (PIECE_SIDE + PIECE_DISTANCE);
      int col = (x - X_OFFSET_PIECES) / (PIECE_SIDE + PIECE_DISTANCE);

      // Dependendo de quem  o proximo jogador, prepara a tela.
      if ( !this.game.currentPlayerIsPalm() )
      {
        this.selection.selectRow( row );
        this.selection.togglePiece( col );
      }
    }

    repaint();
  }

  /**
   * Seta o command apropiado ao jogador atual.
   */
  private void setCurrentPlayerCommand()
  {
    // Se possvel, remove o comando existente da tela.
    if ( this.chooseCommand != null )
    {
      removeCommand( this.chooseCommand );
      this.chooseCommand = null;
    }

    // Se no terminou, prepara a tela dependendo de quem  o proximo jogador.
    if ( !this.game.isTerminated() )
    {
      String newLabel = this.game.currentPlayerIsPalm() ? "Continuar" : "Pegar";
      this.chooseCommand = new Command( newLabel, Command.SCREEN, 2 );
      addCommand( this.chooseCommand );
    }
  }
} // class NimCanvas



