/**
 * Modela um jogo NIM.
 * @stereotype Model
 */
public class NimGame
{
  /**
   * Os jogadores.
   * Como simplificao, assumimos que:
   *
   *  PLAYER_1 --> Operador
   *  PLAYER_2 --> Palm
   *
   * Melhor se usar padro Enumerator.
   */
  public static final int PLAYER_1 = 1;
  public static final int PLAYER_2 = 2;

  /**
   * O expert que sabe resolver NIMs.
   * @supplierRole expert
   * @supplierCardinality 1
   */
  private Expert expert;

  /**
   * O numero de filas.
   */
  static final int MAX_ROWS = 4;

  /**
   * As filas no jogo, de 0 a MAX_ROWS - 1;
   * O valor na posio rows[ i ] significa que na fila i tem rows[ i ] elementos.
   */
  private int[] rows;

  /**
   * Indica quem  o jogador atual.
   */
  private int currentPlayer;

  /**
   * Indica se o jogo acabou.
   */
  private boolean isTerminated;

  /**
   * Indica o mensajem atual a ser mostrado aos jogadores na tela. Pode ser vazio ("").
   */
  private String  currentMessage;


  public NimGame()
  {
    // Initialize structures.
    this.rows = new int[ MAX_ROWS ];

    // No expert yet.
    this.expert = null;

    // Initialize states with a default expert.
    startNewGame( Expert.createInstance( Expert.WINNER_EXPERT ) );
  }


  /**
   * Inicializa um novo jogo.
   */
  public void startNewGame( Expert expert )
  {
    this.expert = expert;

    this.isTerminated = false;

    // Decides that the first player is always the PLAYER_1.
    this.currentPlayer = PLAYER_1;

    // Carrega a configurao default das fichas (Mariembad like).
    for ( int i = 0; i < MAX_ROWS; i++ )
    {
      this.rows[ i ] = (2 * i + 1);
    }

    this.currentMessage = "Sua vez...";
  }


  public String getCurrentMessage()
  {
    return this.currentMessage;
  }


  /**
   * @return O numero de filas no tabuleiro.
   */
  public final int getNumberOfRows()
  {
    return MAX_ROWS;
  }


  /**
   * @param rowNumber O numero de fila, em [0, getNumberOfRows() - 1]
   * @return O nmero de fichas na fila especificada.
   */
  public int getNumberOfPiecesInRow( int rowNumber )
  {
    return this.rows[ rowNumber ];
  }


  /**
   * Informa se o jogador atual  a mquina.
   * til para decidir carateristicas da user interface.
   */
  public boolean currentPlayerIsPalm()
  {
    return( this.currentPlayer != PLAYER_1 );
  }


  /**
   * @return True se o jogo terminou, false caso contrario.
   */
  public boolean isTerminated()
  {
    return this.isTerminated;
  }


  /**
   * Faz uma jogada com o jogador atual.
   * Se o jogo est terminado, lana uma IllegalState exception.
   */
  public Selection play( Selection selection )
  {
    Selection result = null;

    if ( selection.countPieces() == 0 )
    {
      this.currentMessage = "Voc esqueceu de algo?";
      result = selection;
    }
    else
    {
      // Play with current player.
      result = play( this.currentPlayer, selection );

      // Change players.
      this.currentPlayer = ( this.currentPlayer == PLAYER_1 ) ? PLAYER_2 : PLAYER_1;

      // Decides the state of game.
      this.isTerminated = isAllEmpty();

      // Decides new message.
      if ( !this.isTerminated )
      {
        this.currentMessage = ( this.currentPlayer == PLAYER_1 ) ?
                              "Sua vez..." :
                              "Esta  minha jogada: (" +
          result.getSelectedRow() + "," + result.countPieces() + ")";
      }
      else
      {
        this.currentMessage = ( this.currentPlayer == PLAYER_1 ) ?
                              "Voc perdeu >:-(" :
                              "Voc ganhou :-D"  ;
      }
    }

    return result;
  }


  /**
   * Faz uma jogada para o usurio especificado.
   * Se o jogo est terminado, lana uma IllegalState exception.
   *
   * @return Se o player for PLAYER_2, retorna uma Selection vazia.
   *         Se for PLAYER_1, retorna a jogada do PLAYER_2.
   */
  private Selection play( int player, Selection selection )
  {
    if ( this.isTerminated )
      throw new IllegalStateException( "No pode jogar se o jogo terminou!" );

    Selection result = new Selection();

    applySelection( selection );

    if ( player == PLAYER_1 )  // Human.
    {
      this.expert.computerTurn( this.rows, result );
    }
    else if ( player == PLAYER_2 )
    {
      // Nothing to do.
    }
    else // Se usasemos enumerator, no seria necessaria esta linha!!!!!!! @todo
      throw new IllegalArgumentException( "Jogador inesperado: " + player );

    return result;
  }

  /**
   * @return True se no tem mais peas no tabuleiro.
   */
  private boolean isAllEmpty()
  {
    boolean result = true;

    for ( int i = 0; i < this.rows.length; i++ )
    {
      if ( this.rows[ i ] != 0 )
      {
        result = false;
        break;
      }
    }

    return result;
  }

  /**
   * Aplica a Selection especificada.
   */
  private void applySelection( Selection selection )
  {
    this.rows[ selection.getSelectedRow() ] -= selection.countPieces();
  }

} // class NimGame


